@@ -144,6 +144,10 @@ extern "C" {
 #  define MRN_OPTION_BITS_IS_UNDER_VARIABLES
 #endif
 
+#if MYSQL_VERSION_ID >= 50600
+#  define MRN_HAVE_HA_REBIND_PSI
+#endif
+
 class ha_mroonga;
 
 /* structs */
@@ -447,8 +451,19 @@ protected:
   uint referenced_by_foreign_key();
   void init_table_handle_for_HANDLER();
   void free_foreign_key_create_info(char* str);
+#ifdef MRN_HAVE_HA_REBIND_PSI
+  void unbind_psi();
+  void rebind_psi();
+#endif
+  my_bool register_query_cache_table(THD *thd,
+                                     char *table_key,
+                                     uint key_length,
+                                     qc_engine_callback *engine_callback,
+                                     ulonglong *engine_data);
 
 private:
+  void mkdir_p(const char *directory);
+
   void push_warning_unsupported_spatial_index_search(enum ha_rkey_function flag);
   void clear_cursor();
   void clear_cursor_geo();
@@ -576,11 +591,13 @@ private:
                                     int i,
                                     KEY *key_info,
                                     grn_obj **index_tables,
+                                    grn_obj **index_columns,
                                     MRN_SHARE *tmp_share);
   int wrapper_create_index_geo(const char *grn_table_name,
                                int i,
                                KEY *key_info,
                                grn_obj **index_tables,
+                               grn_obj **index_columns,
                                MRN_SHARE *tmp_share);
   int wrapper_create_index(const char *name, TABLE *table,
                            HA_CREATE_INFO *info, MRN_SHARE *tmp_share,
@@ -594,6 +611,7 @@ private:
   int storage_create_indexes(TABLE *table, const char *grn_table_name,
                              grn_obj *grn_table, MRN_SHARE *tmp_share);
   int close_databases();
+  void ensure_database_directory();
   int ensure_database_create(const char *name);
   int ensure_database_open(const char *name);
   int wrapper_delete_table(const char *name, MRN_SHARE *tmp_share,
@@ -948,6 +966,24 @@ private:
   void wrapper_set_keys_in_use();
   void storage_set_keys_in_use();
   bool check_written_by_row_based_binlog();
+#ifdef MRN_HAVE_HA_REBIND_PSI
+  void wrapper_unbind_psi();
+  void storage_unbind_psi();
+  void wrapper_rebind_psi();
+  void storage_rebind_psi();
+#endif
+  my_bool wrapper_register_query_cache_table(THD *thd,
+                                             char *table_key,
+                                             uint key_length,
+                                             qc_engine_callback
+                                             *engine_callback,
+                                             ulonglong *engine_data);
+  my_bool storage_register_query_cache_table(THD *thd,
+                                             char *table_key,
+                                             uint key_length,
+                                             qc_engine_callback
+                                             *engine_callback,
+                                             ulonglong *engine_data);
 };
 
 #ifdef __cplusplus
