@@ -7,47 +7,37 @@ if test "$NO_MAKE" != "yes"; then
     make -C ${top_dir} > /dev/null || exit 1
 fi
 
-if test -z "$MYSQL_SOURCE"; then
-    MYSQL_SOURCE="$(make -s -C $top_dir echo-mysql-source)"
-fi
-export MYSQL_SOURCE
-
-if test -z "$MYSQL_BUILD"; then
-    MYSQL_BUILD="$(make -s -C $top_dir echo-mysql-build)"
-fi
-export MYSQL_BUILD
-
-if test -z "$MYSQL_VERSION"; then
-    MYSQL_VERSION="$(make -s -C $top_dir echo-mysql-version)"
-fi
-export MYSQL_VERSION
+. "${top_dir}/config.sh"
 
 test_suite_names="mroonga_storage,mroonga_wrapper"
-source_mysql_test_dir="${MYSQL_SOURCE}/mysql-test"
-build_mysql_test_dir="${MYSQL_BUILD}/mysql-test"
+source_mysql_test_dir="${MYSQL_SOURCE_DIR}/mysql-test"
+build_mysql_test_dir="${MYSQL_BUILD_DIR}/mysql-test"
 source_test_suites_dir="${source_mysql_test_dir}/suite"
 build_test_suites_dir="${build_mysql_test_dir}/suite"
 build_test_include_dir="${build_mysql_test_dir}/include"
 case "${MYSQL_VERSION}" in
     5.1.*)
-	plugins_dir="${MYSQL_BUILD}/lib/mysql/plugin"
+	plugins_dir="${MYSQL_BUILD_DIR}/lib/mysql/plugin"
 	if ! test -d "${build_test_suites_dir}"; then
 	    mkdir -p "${build_test_suites_dir}"
 	fi
 	;;
-    *-MariaDB*)
-	if ! test -d "${build_test_suites_dir}"; then
-	    ln -s "${source_test_suites_dir}" "${build_test_suites_dir}"
-	fi
-	if ! test -d "${MYSQL_BUILD}/plugin/mroonga"; then
-	    ln -s "${top_dir}" "${MYSQL_BUILD}/plugin/mroonga"
-	fi
-	;;
     *)
 	if ! test -d "${build_test_suites_dir}"; then
 	    ln -s "${source_test_suites_dir}" "${build_test_suites_dir}"
 	fi
-	plugins_dir="${MYSQL_SOURCE}/lib/plugin"
+	maria_storage_dir="${MYSQL_SOURCE_DIR}/storage/maria"
+	if test -d "${maria_storage_dir}"; then
+	    mariadb="yes"
+	else
+	    mariadb="no"
+	fi
+	if test "${mariadb}" = "yes"; then
+	    ln -s "${top_dir}" "${MYSQL_BUILD_DIR}/plugin/mroonga"
+	    plugins_dir=
+	else
+	    plugins_dir="${MYSQL_SOURCE_DIR}/lib/plugin"
+	fi
 	;;
 esac
 
@@ -87,34 +77,46 @@ done
 
 innodb_test_suite_dir="${build_test_suites_dir}/innodb"
 mroonga_wrapper_innodb_test_suite_dir="${build_test_suites_dir}/mroonga_wrapper_innodb"
+mroonga_wrapper_innodb_include_dir="${mroonga_wrapper_innodb_test_suite_dir}/include/"
 if test "$0" -nt "$(dirname "${mroonga_wrapper_innodb_test_suite_dir}")"; then
     rm -rf "${mroonga_wrapper_innodb_test_suite_dir}"
 fi
 if ! test -d "${mroonga_wrapper_innodb_test_suite_dir}"; then
     cp -rp "${innodb_test_suite_dir}" "${mroonga_wrapper_innodb_test_suite_dir}"
+    mkdir -p "${mroonga_wrapper_innodb_include_dir}"
+    cp -rp "${build_test_include_dir}"/innodb[-_]*.inc \
+	"${mroonga_wrapper_innodb_include_dir}"
     ruby -i'' \
-	-pe "\$_.gsub!(/\bengine\s*=\s*innodb\b([^;\\n]*)/i,
-                      \"ENGINE=mroonga\\\1 COMMENT='ENGINE \\\"InnoDB\\\"'\")
+	-pe "\$_.gsub!(/\\bengine\\s*=\\s*innodb\\b([^;\\n]*)/i,
+                       \"ENGINE=mroonga\\\1 COMMENT='ENGINE \\\"InnoDB\\\"'\")
+             \$_.gsub!(/\\b(storage_engine\\s*=\\s*)innodb\\b([^;\\n]*)/i,
+                       \"\\\1mroonga\")
+             \$_.gsub!(/^(--\\s*source\\s+)(include\\/innodb)/i,
+                       \"\\\1suite/mroonga_wrapper_innodb/\\\2\")
             " \
 	${mroonga_wrapper_innodb_test_suite_dir}/r/*.result \
 	${mroonga_wrapper_innodb_test_suite_dir}/t/*.test \
-	${mroonga_wrapper_innodb_test_suite_dir}/include/*.inc \
-	${build_test_include_dir}/innodb_*.inc # over our work. :<
+	${mroonga_wrapper_innodb_test_suite_dir}/include/*.inc
     sed -i'' \
 	-e '1 i --source include/have_mroonga.inc' \
 	${mroonga_wrapper_innodb_test_suite_dir}/t/*.test
 fi
 
 if test -n "${plugins_dir}"; then
-    make -C ${top_dir} \
-	install-pluginLTLIBRARIES \
-	plugindir=${plugins_dir} > /dev/null || \
-	exit 1
+    if test -d "${top_dir}/.libs"; then
+	make -C ${top_dir} \
+	    install-pluginLTLIBRARIES \
+	    plugindir=${plugins_dir} > /dev/null || \
+	    exit 1
+    else
+	cp "${top_dir}/ha_mroonga.so" "${plugins_dir}" || exit 1
+    fi
 fi
 
 (cd "$build_mysql_test_dir" && \
     ./mysql-test-run.pl \
     --no-check-testcases \
+    --parallel=4 \
     --retry=1 \
     --suite="${test_suite_names}" \
     --force \
